@@ -10,9 +10,9 @@ def ei(x, model, params):
     """
 
     minacq = np.min(model.Y)
-    m, v = model.predict_noiseless(np.atleast_2d(x))
+    m, v = model.predict(np.atleast_2d(x), noise=False)
     s = np.sqrt(v)
-    dmdx, dvdx = model.predictive_gradients(np.atleast_2d(x))
+    dmdx, dvdx = model.predict_grads(np.atleast_2d(x))
     dmdx = dmdx[:, :, 0]
     z = (minacq - m) / s
     phi = scipy.stats.norm.pdf(z)
