@@ -7,7 +7,7 @@ def exploit(x, model, params):
 
     Doesn't take any parameters (apart from the model).
     """
-    m, s, dmdx, dsdx = model.predictive_m_s_grad(x)
+    m, s, dmdx, dsdx = model.predict_mean_sd_grads(x)
     f_acqu = m
     df_acqu = dmdx
     scipygradient = np.asmatrix(df_acqu).transpose()
