@@ -9,7 +9,7 @@ def lcb(x, model, params):
     """
 
     explr_weight = params[0]
-    m, s, dmdx, dsdx = model.predictive_m_s_grad(x)
+    m, s, dmdx, dsdx = model.predict_mean_sd_grads(x)
     f_acqu = m - explr_weight * s
     df_acqu = dmdx - explr_weight * dsdx
     scipygradient = np.asmatrix(df_acqu).transpose()
