@@ -1,6 +1,5 @@
 import os
 
-import GPy
 import numpy as np
 
 from pathlib import Path
@@ -94,30 +93,29 @@ class BOMain:
         self.minzacc = settings["minzacc"]
         self.kerntype = settings["kernel"]
         self.noise = settings["noise"]
+        self.ynorm = settings["ynorm"]
         self.timer = Timer()
         self.dir = settings.dir
         self.gm_tol = settings["gm_tol"]
 
         self.kernel = KernelFactory.construct_kernel(settings)
         self.model = None
-        self.initX = np.array([]).reshape(0, self.dim)
-        self.initY = np.array([]).reshape(0, 1)
+        self.X_init = np.array([]).reshape(0, self.dim)
+        self.Y_init = np.array([]).reshape(0, 1)
         self.convergence = np.array([]).reshape(0, 5)
-        self.normmean = 0
-        self.normsd = 1
         self.est_yrange = 0
         self.hmcsample = []
-
-        self.unfixed_model_params = np.array([]).reshape(0, self.dim + 1)
         self.exit_ok = False  # successful exit from main loop
 
+        Minimization.set_parallel(self.settings["parallel_optims"])
+
     def get_x(self):
         """
         Returns the points where the objective function has been evaluated,
         in order of acquisition.
         """
         if self.model == None:
-            return self.initX
+            return self.X_init
         else:
             return self.model.X
 
@@ -128,68 +126,67 @@ class BOMain:
         BOMain.model.Y for any use outside of the BOMain class.
         """
         if self.model == None:
-            return self.initY
+            return self.Y_init
         else:
-            return self.model.Y * self.normsd + self.normmean
+            return self.model.Y
+
+    def get_mu_and_nu(self, x):
+        """
+        Returns the model prediction (mu) and its standard deviation (nu)
+        at point x.
+        The standard deviation is calculated based on predictive variance
+        with noise (model variance).
+        """
+        if self.model is None:
+            return None, None
+        else:
+            mu, var = self.model.predict(x)
+            nu = np.sqrt(var)
+            return mu, nu
 
     def get_mu(self, x):
         """
-        Returns the model prediction at point x. This method should always be
-        called instead of BOMain.model.mu for any use outside of the BOMain
-        class.
+        Returns the model prediction at point x.
         """
-        if self.model == None:
-            return None
-        else:
-            return self.model.mu(x) * self.normsd + self.normmean
+        return self.get_mu_and_nu(x)[0]
 
     def get_nu(self, x):
         """
-        Returns the variance of the model prediction at point x, with added
-        noise (model variance).
+        Returns the standard deviation of the model prediction at point x,
+        calculated based on predictive variance with noise (model variance).
         """
-        if self.model == None:
-            return None
-        else:
-            return self.model.nu(x) * self.normsd
+        return self.get_mu_and_nu(x)[1]
 
     def get_grad(self, x):
         """
-        Returns the predictive gradients. If the implemented mean shift is
-        a constant, should just wrap self.model.predictive_gradients.
+        Returns the predictive gradients.
         """
-        g = self.model.predictive_gradients(x)
-        return np.array([g[0] * self.normsd, None])
+        g = self.model.predict_grads(x)
+        return np.array([g[0], None])
 
-    def _add_xy(self, xnew, ynew):
+    def _add_xy(self, x_new, y_new):
         """
-        Internal functionality for saving a new acquisition (x, y), accounting
-        for model mean shift. Initializes the GP model when the number of
-        acquisitions meets the pre-specified number of initialization points.
+        Internal functionality for saving a new acquisition (x, y).
+        Initializes the GP model when the number ofacquisitions meets the
+        pre-specified number of initialization points.
         """
         if self.model == None:
-            self.initX = np.vstack([self.initX, xnew])
-            self.initY = np.vstack([self.initY, ynew])
+            self.X_init = np.vstack([self.X_init, x_new])
+            self.Y_init = np.vstack([self.Y_init, y_new])
             if self.get_x().shape[0] == self.initpts:
                 self._init_model()
         else:
-            X = np.vstack([self.get_x(), np.atleast_2d(xnew)])
-            Y = np.vstack([self.get_y(), ynew])
-            self.normmean = np.mean(Y)
-            # if self.settings['ynorm']:
-            #     self.normsd = np.std(Y)  # NORMALIZATION
-            self.normsd = 1.0
-            self.est_yrange = np.max(Y) - np.min(Y)
-            self.model.redefine_data(X, (Y - self.normmean) / self.normsd)
+            self.model.add_data(x_new, y_new)
+            self.est_yrange = np.max(self.model.Y) - np.min(self.model.Y)
 
-    def add_xy_list(self, xnew, ynew):
+    def add_xy_list(self, x_new, y_new):
         """
-        Saves multiple acquisitions, accounting for model mean shift.
-        Initializes the GP model when the number of acquisitions meets the
-        pre-specified number of initialization points.
+        Saves multiple acquisitions. Initializes the GP model when the number
+        of acquisitions meets the pre-specified number of initialization
+        points.
         """
-        for i in range(xnew.shape[0]):
-            self._add_xy(xnew[i, :], ynew[i])
+        for i in range(x_new.shape[0]):
+            self._add_xy(x_new[i, :], y_new[i])
 
     def _init_model(self):
         """
@@ -197,22 +194,16 @@ class BOMain:
         points have been evaluated. Further acquisition locations can then be
         queried by optimizing an acquisition function.
         """
-        self.normmean = np.mean(self.initY)
-        # if self.settings['ynorm']:
-        #     self.normsd = np.std(self.initY)  # NORMALIZATION
-        self.normsd = 1.0
-        self.est_yrange = np.max(self.initY) - np.min(self.initY)
+        self.est_yrange = np.max(self.Y_init) - np.min(self.Y_init)
         self.model = Model(
-            self.initX,
-            (self.initY - self.normmean) / self.normsd,
-            self.bounds,
-            self.min_dist_acqs,
-            self.minzacc,
-            self.kerntype,
+            self.X_init,
+            self.Y_init,
             self.kernel,
-            self.dim,
             self.noise,
+            self.ynorm,
         )
+        n = len(self.model.get_unfixed_params())
+        self.unfixed_model_params = np.array([]).reshape(0, n)
 
     def _should_optimize(self, i_iter):
         """
@@ -253,8 +244,13 @@ class BOMain:
             An object that provides convenient access to the most
             important results from the optimization.
         """
-        self.main_output.new_file()
-        self.rst_manager.new_file()
+        self.exit_ok = False
+
+        # Resolve current iteration and set optimisation parameters
+        i_current = len(self.get_x())
+        if i_current == 0:
+            self.main_output.new_file()
+            self.rst_manager.new_file()
 
         initpts = self.settings["initpts"]
         iterpts = self.settings["iterpts"]
@@ -262,29 +258,29 @@ class BOMain:
 
         totalpts = initpts + iterpts
 
-        xnext = self._get_xnext(0)
+        x_next = self._get_x_next(i_current)
 
         # BO main loop
-        for i_iter in range(totalpts):
+        for i_iter in np.arange(i_current, totalpts):
 
             self.main_output.iteration_start(i_iter + 1)
             self.timer.startLap()
 
             # Evaluation
-            xnew, ynew = self._evaluate(i_iter, xnext)
+            x_new, y_new = self._evaluate(i_iter, x_next)
 
             # Store new data and refit model.
             #  - Create the model when all initial data has been acquired.
-            for i in range(len(ynew)):
-                self._add_xy(xnew[i], ynew[i])
-                self.rst_manager.new_data(xnew[i], ynew[i])
+            for i in range(len(y_new)):
+                self._add_xy(x_new[i], y_new[i])
+                self.rst_manager.new_data(x_new[i], y_new[i])
 
             # Optimize model if needed.
             if self._should_optimize(i_iter):
                 self._optimize_model(i_iter)
 
             # Find next acquisition point.
-            xnext = self._get_xnext(i_iter + 1)
+            x_next = self._get_x_next(i_iter + 1)
 
             # Add model parameters to rst-file.
             if self.model != None:
@@ -311,10 +307,10 @@ class BOMain:
             self.main_output.iteration_summary(
                 i_iter,
                 self.get_y().size,
-                xnew,
-                ynew,
+                x_new,
+                y_new,
                 self.convergence,
-                xnext,
+                x_next,
                 self.est_yrange,
                 mod_unfixed_par,
                 self.timer,
@@ -353,71 +349,8 @@ class BOMain:
             A dict-like object that provides convenient access to the most
             important results from the optimization.
         """
-        i_curr = self.model.X.shape[0]
-        totalpts = iterpts + self.settings["initpts"]
-        minfreq = self.settings["minfreq"]
         self.settings["iterpts"] = iterpts
-
-        for i_iter in range(i_curr + 1, totalpts + 1):
-
-            self.main_output.iteration_start(i_iter)
-            self.timer.startLap()
-
-            # Acquisition
-            xnext, acqfn = self._acqnext(i_iter)
-
-            # Evaluation
-            xnew, ynew = self._evaluate_xnew(xnext)
-
-            # Store new data and refit model.
-            #  - Create the model when all initial data has been acquired.
-            for i in range(len(ynew)):
-                self._add_xy(xnew[i], ynew[i])
-                self.rst_manager.new_data(xnew[i], ynew[i])
-
-            # Optimize model if needed.
-            if self._should_optimize(i_iter):
-                self._optimize_model(i_iter)
-
-            # Add model parameters to rst-file.
-            if self.model != None:
-                mod_unfixed_par = self.model.get_unfixed_params()
-                self.rst_manager.new_model_params(mod_unfixed_par)
-
-                # Also store model parameters directly on BOMain
-                self.unfixed_model_params = np.vstack(
-                    (self.unfixed_model_params, mod_unfixed_par)
-                )
-            else:
-                mod_unfixed_par = None
-
-            # Convergence diagnostics, calculate iteration specific info
-            if (minfreq > 0 and i_iter % minfreq == 0) or i_iter == totalpts:
-                self._add_convergence()
-
-            # Output to boss outfile
-            # Print a summary of important iteration data
-            self.main_output.iteration_summary(
-                i_iter,
-                self.get_y().size,
-                xnew,
-                ynew,
-                self.convergence,
-                xnext,
-                self.est_yrange,
-                mod_unfixed_par,
-                self.timer,
-            )
-
-            # Stopping criterion: convergence
-            if self._has_converged(i_iter):
-                self.main_output.convergence_stop()
-                break
-
-        # If we get here, the optimization finished successfully.
-        self.exit_ok = True
-        res = self.get_results()
-        return res
+        return self.run()
 
     def get_results(self):
         return BOResults(self)
@@ -438,24 +371,24 @@ class BOMain:
                 return within_tol
         return False
 
-    def _get_xnext(self, i_iter):
+    def _get_x_next(self, i_iter):
         """
         Get a new point to evaluate by either reading it from the rst-file or,
         in case it doesn't contain the next point to evaluate, by obtaining
         a new initial point (when run is in initialization stage) or
         minimizing the acquisition function (when the run is in BO stage).
         """
-        xnext = self.rst_manager.get_x(i_iter)
-        if np.any(xnext == None):
+        x_next = self.rst_manager.get_x(i_iter)
+        if np.any(x_next == None):
             if i_iter < self.initpts:
-                xnext = self.init_manager.get_x(i_iter)
+                x_next = self.init_manager.get_x(i_iter)
             else:
-                xnext, acqfn = self._acqnext(i_iter)
-        return xnext
+                x_next, acqfn = self._acqnext(i_iter)
+        return x_next
 
     def _acqnext(self, i_iter):
         """
-        Selects the acquisition function to use and returns its xnext location
+        Selects the acquisition function to use and returns its x_next location
         as well as the used acquisition function.
         """
         if self.hmciters == 0:
@@ -466,50 +399,34 @@ class BOMain:
             acqfn = hmc.averageacq(self.acqfn, self.cores)
             expfn = hmc.averageacq(explore, self.cores)
 
-        xnext = self._minimize_acqfn(acqfn)
+        x_next = self._minimize_acqfn(acqfn)
 
-        # check if xnext indicates we should trigger pure exploration
-        if self._location_overconfident(xnext):
-            xnext = self._minimize_acqfn(expfn)
-            return xnext, expfn
+        # check if x_next indicates we should trigger pure exploration
+        if self._location_overconfident(x_next):
+            x_next = self._minimize_acqfn(expfn)
+            return x_next, expfn
         else:
-            return xnext, acqfn
+            return x_next, acqfn
 
     def _minimize_acqfn(self, acqfn):
         """
         Minimizes the acquisition function to find the next
-        sampling location 'xnext'.
+        sampling location 'x_next'.
         """
-
-        nof_stdp = len(np.where(np.array(self.kerntype) == "stdp")[0])
-
         # Calculate the number of local minimizers to start. This has two
         # steps:
 
         # 1. Estimate the number of local minima in the surrogate model.
-        # For the ith dimension, the number of local minima along a slice
-        # is approximately n(i) = boundlength(i)/(2*lengthscale(i)). Note
-        # that periodic kernels operate on normalised distances: distance
-        # between inputs that are period(i)/2 apart is 1. To get the total
-        # number of minima for all of the search space, multiply together
-        # n(i) over all i.
-        lengthscale_numpts = 1
-        for i in range(self.dim):
-            lengthscale = self.model.get_unfixed_params()[i + 1]
-            if self.kerntype[i] == "stdp":
-                bound_distance = 1
-            else:
-                bound_distance = self.periods[i] / 2
-            lengthscale_numpts *= max(1, bound_distance / lengthscale)
+        estimated_numpts = self.model.estimate_num_local_minima(self.bounds)
 
         # 2. Increase the estimate to approximate the number of local
         # minima in the acquisition function. Here we assume that the
         # increase is proportional to the estimated number of local minima
         # per dimension.
         minima_multiplier = 1.7
-        lengthscale_numpts = (minima_multiplier ** self.dim) * lengthscale_numpts
+        estimated_numpts = (minima_multiplier ** self.dim) * estimated_numpts
 
-        num_pts = min(len(self.model.X), int(lengthscale_numpts))
+        num_pts = min(len(self.model.X), int(estimated_numpts))
 
         # minimize acqfn to obtain sampling location
         gmin = Minimization.minimize_from_random(
@@ -521,14 +438,15 @@ class BOMain:
         )
         return np.atleast_1d(np.array(gmin[0]))
 
-    def _location_overconfident(self, xnext):
+    def _location_overconfident(self, x_next):
         """
-        Checks is model variance is lower than tolerance at suggested xnext.
+        Checks is model variance is lower than tolerance at suggested x_next.
         """
         if self.acqtol is None:
             return False
         else:
-            if self.model.nu(xnext) >= self.acqtol:
+            nu_x_next = np.sqrt(self.model.predict(x_next)[1])
+            if nu_x_next >= self.acqtol:
                 return False
             else:
                 self.main_output.progress_msg(
@@ -536,45 +454,40 @@ class BOMain:
                 )
                 return True
 
-    def _evaluate(self, i_iter, xnext):
+    def _evaluate(self, i_iter, x_next):
         """
         Get a new evaluation either from the rst-file or, in case it doesn't
         contain the corresponding evaluation, by evaluating the user function
         """
-        ynew = self.rst_manager.get_y(i_iter)
-        if np.any(ynew == None):
-            return self._evaluate_xnew(xnext)
+        y_new = self.rst_manager.get_y(i_iter)
+        if np.any(y_new == None):
+            return self._evaluate_x_new(x_next)
         else:
-            return np.atleast_2d(xnext), np.atleast_1d(ynew)
+            return np.atleast_2d(x_next), np.atleast_1d(y_new)
 
-    def _evaluate_xnew(self, xnew):
+    def _evaluate_x_new(self, x_new):
         """
-        Evaluates user function at given location 'xnew'
-        to get the observation scalar 'ynew'.
+        Evaluates user function at given location 'x_new'
+        to get the observation scalar 'y_new'.
         Later also gradient 'ydnew' should be made possible.
         """
         # run user script to evaluate sample
         self.main_output.progress_msg(
             "Evaluating objective function at x ="
-            + self.main_output.utils.oneDarray_line(xnew.flatten(), self.dim, float),
+            + self.main_output.utils.oneDarray_line(x_new.flatten(), self.dim, float),
             1,
         )
 
         local_timer = Timer()
-        xnew = np.atleast_2d(xnew)  # X-data expected to be 2d-array
-        ynew = np.atleast_1d(self.f(xnew))  # call the user function
+        x_new = np.atleast_2d(x_new)  # X-data expected to be 2d-array
+        y_new = np.atleast_1d(self.f(x_new))  # call the user function
         # os.chdir(self.dir)  # in case the user function changed the dir
         self.main_output.progress_msg(
             "Objective function evaluated,"
             + " time [s] %s" % (local_timer.str_lapTime()),
             1,
         )
-
-        # return new data
-        for i in range(len(xnew) - 1):
-            ynew = np.insert(ynew, -1, ynew[0])
-
-        return xnew, ynew
+        return x_new, y_new
 
     def _optimize_model(self, i_iter):
         """
@@ -582,18 +495,14 @@ class BOMain:
         the rst-file, just use them.
         """
         if self.hmciters > 0:
-            hmc = GPy.inference.mcmc.HMC(self.model)
-            burnin = hmc.sample(int(self.hmciters * 0.33))
-            self.hmcsample = hmc.sample(self.hmciters)
+            self.hmcsample = self.model.sample_unfixed_params(self.hmciters)
 
         n = self.model.get_unfixed_params().size
         theta = self.rst_manager.get_theta(i_iter, n)
         if np.any(theta == None):
-            self.model.optimize_controlrstr(self.updaterestarts)
+            self.model.optimize(self.updaterestarts)
         else:
             self.model.set_unfixed_params(theta)
-            # This is required for the model to properly update.
-            self.model.redefine_data(self.model.X, self.model.Y)
 
     def _add_convergence(self):
         """
@@ -603,9 +512,24 @@ class BOMain:
         # if self.model == None:
         #     conv = np.atleast_2d(np.repeat(np.nan, 5))
         # else:
-        bestx, besty = self.model.get_bestxy()
-        xhat, muhat, nuhat = self.model.min_prediction()
-        besty = besty * self.normsd + self.normmean
-        muhat = muhat * self.normsd + self.normmean
+        bestx, besty = self.model.get_best_xy()
+        xhat, muhat, nuhat = self._min_model_prediction()
         conv = np.atleast_2d([bestx, besty, xhat, muhat, nuhat])
         self.convergence = np.vstack((self.convergence, conv))
+
+    def _min_model_prediction(self):
+        """
+        Finds and returns the model global minimum prediction xhat
+        and the model mean (muhat) and variance (nuhat) at that point.
+        """
+        gmin = Minimization.minimize(
+            self.model.predict_mean_grad,
+            self.bounds,
+            self.kerntype,
+            np.hstack([self.model.X, self.model.Y]),
+            self.min_dist_acqs,
+            accuracy=self.minzacc,
+        )
+        xhat = np.array(gmin[0])
+        muhat, nuhat = self.model.predict(np.atleast_2d(xhat))
+        return xhat, muhat[0][0], nuhat[0][0]
