@@ -7,12 +7,11 @@ import numpy as np
 import matplotlib.pyplot as plt
 
 plt.switch_backend("agg")
-import warnings
 
 
 def plot_data_acquisitions(settings, dest_file, acqs, min_preds):
     """
-    Plots minimum predictions and uncertainty, acquisition energies 
+    Plots minimum predictions and uncertainty, acquisition energies
     and locations as a function of iteration.
     """
     itNo_acqs = np.clip(acqs[:, 0], int(min_preds[0, 0]), np.inf)
@@ -183,7 +182,6 @@ def plot_hyperparameters(settings, dest_file, hypers, legends=True):
 
     plt.suptitle("Hyperparameter values", size=20)
     plt.gcf().set_size_inches(11, 7)
-    # plt.tight_layout(rect=[0, 0.03, 1, 0.95]); plt.savefig(dest_file)
     if legends:  # FOR OLD MATPLOTLIB COMPATIBILITY
         plt.savefig(dest_file, bbox_extra_artists=(lgd,), bbox_inches="tight")
     else:
@@ -228,7 +226,6 @@ def plot_truef_hat(settings, dest_file, truef_hats, legends=True):
 
     plt.suptitle("True function at minimum predictions", size=20)
     plt.gcf().set_size_inches(9, 9)
-    # plt.tight_layout(rect=[0, 0.03, 1, 0.95]); plt.savefig(dest_file)
     if legends:  # FOR OLD MATPLOTLIB COMPATIBILITY
         plt.savefig(dest_file, bbox_extra_artists=(lgd,), bbox_inches="tight")
     else:
@@ -345,7 +342,6 @@ def plot_model(
         plt.gcf().set_size_inches(10, 8)
         plt.gca().tick_params(labelsize=18)
         plt.gca().ticklabel_format(useOffset=False)
-        # plt.tight_layout()
         if legends:  # FOR OLD MATPLOTLIB COMPATIBILITY
             plt.savefig(dest_file, bbox_extra_artists=(lgd,), bbox_inches="tight")
         else:
@@ -353,134 +349,26 @@ def plot_model(
         plt.close()
 
     elif slice_dim == 2:
-        npts = settings["pp_model_slice"][2]
-        x1 = coords[:, settings["pp_model_slice"][0]]
-        x2 = coords[:, settings["pp_model_slice"][1]]
-
-        if truef is not None:
-            pass  # 2D true func slice will be plotted in separate graph only
 
         if incl_uncert:
-            plt.contour(x1[:npts], x2[::npts], nu.reshape(npts, npts), 25, colors="k")
-            plt.contourf(
-                x1[:npts], x2[::npts], nu.reshape(npts, npts), 150, cmap="viridis"
-            )
-            cbar = plt.colorbar()  # , orientation='horizontal')
-            cbar.set_label(label="$\\nu(x)$", size=24)
-            cbar.ax.tick_params(labelsize=18)
-            plt.xlabel(axis_labels[0], size=24)
-            plt.ylabel(axis_labels[1], size=24)
-            plt.gcf().set_size_inches(10, 8)
-            plt.gca().tick_params(labelsize=18)
+            x1, x2, nu = _get_2d_arrays(coords, settings, nu)
+            _plot_2d_contours(x1, x2, nu, label=r'$\nu(x)$')
+            _plot_2d_labels_settings(axis_labels)
             plt.tight_layout()
             plt.savefig(dest_file[:-4] + "_uncert.png")
             plt.close()
 
-        plt.contour(x1[:npts], x2[::npts], mu.reshape(npts, npts), 25, colors="k")
-        plt.contourf(x1[:npts], x2[::npts], mu.reshape(npts, npts), 150, cmap="viridis")
-        cbar = plt.colorbar()  # , orientation='horizontal')
-        cbar.set_label(label="$\mu(x)$", size=24)
-        cbar.ax.tick_params(labelsize=18)
+        x1, x2, mu = _get_2d_arrays(coords, settings, mu)
+        _plot_2d_contours(x1, x2, mu, label=r'$\mu(x)$')
 
         lo = False
-        if xhat is not None:
-            plt.plot(
-                xhat[settings["pp_model_slice"][0]],
-                xhat[settings["pp_model_slice"][1]],
-                "r*",
-                markersize=26,
-                zorder=21,
-                label="$\hat{x}$",
-            )
-            lo = True
+        lo |= _plot_2d_acqs(acqs, settings)
+        lo |= _plot_2d_minima(minima, settings)
+        lo |= _plot_2d_xnext(xnext, settings)
+        lo |= _plot_2d_xhat(xhat, settings)
+        _plot_2d_paths(paths, settings)
 
-        if acqs is not None:
-            x1 = acqs[:, settings["pp_model_slice"][0]]
-            x2 = acqs[:, settings["pp_model_slice"][1]]
-            sz = np.linspace(200, 500, len(x1))
-            lw = np.linspace(3, 8, len(x1))
-            plt.scatter(
-                x1[0],
-                x2[0],
-                s=sz[int(len(x1) / 2.0)],
-                linewidth=lw[int(len(x1) / 2.0)],
-                zorder=10,
-                facecolors="none",
-                edgecolors="magenta",
-                label="acqs",
-            )
-            for i in range(len(x1)):
-                plt.scatter(
-                    x1[i],
-                    x2[i],
-                    s=sz[i],
-                    linewidth=lw[i],
-                    zorder=10,
-                    facecolors="none",
-                    edgecolors="magenta",
-                )
-            lo = True
-
-        if xnext is not None:
-            plt.plot(
-                xnext[settings["pp_model_slice"][0]],
-                xnext[settings["pp_model_slice"][1]],
-                "b^",
-                markersize=26,
-                label="$x_{next}$",
-                zorder=20,
-            )
-            lo = True
-
-        if minima is not None:
-            x1 = minima[:, settings["pp_model_slice"][0]]
-            x2 = minima[:, settings["pp_model_slice"][1]]
-            plt.scatter(
-                x1,
-                x2,
-                s=350,
-                linewidth=6,
-                facecolors="none",
-                edgecolors="navajowhite",
-                zorder=11,
-                label="minima",
-            )
-            lo = True
-
-        if paths is not None:
-            threshold = min(settings["bounds"][:, 1] - settings["bounds"][:, 0]) / 2
-            for path in paths:
-                start = 0
-                stop = 1
-                while True:
-                    if (
-                        stop == path.crds.shape[0] - 1
-                        or np.linalg.norm(path.crds[stop, :] - path.crds[stop - 1, :])
-                        > threshold
-                    ):
-                        plt.plot(
-                            path.crds[range(start, stop), 0],
-                            path.crds[range(start, stop), 1],
-                            linewidth=3.0,
-                            color="red",
-                        )
-                        if stop == path.crds.shape[0] - 1:
-                            break
-                        else:
-                            start = stop
-                    stop += 1
-
-        plt.xlim(
-            min(coords[:, settings["pp_model_slice"][0]]),
-            max(coords[:, settings["pp_model_slice"][0]]),
-        )
-        plt.ylim(
-            min(coords[:, settings["pp_model_slice"][1]]),
-            max(coords[:, settings["pp_model_slice"][1]]),
-        )
-        plt.xlabel(axis_labels[0], size=24)
-        plt.ylabel(axis_labels[1], size=24)
-        top = 0.99
+        _plot_2d_labels_settings(axis_labels)
         if legends and lo:
             lgd = plt.legend(
                 bbox_to_anchor=(0.0, 1.02, 1.0, 0.102),
@@ -490,11 +378,6 @@ def plot_model(
                 borderaxespad=0.0,
                 prop={"size": 20},
             )
-            top = 0.85
-        plt.gcf().set_size_inches(10, 8)
-        plt.gca().tick_params(labelsize=18)
-        plt.gca().ticklabel_format(useOffset=False)
-        # plt.tight_layout()
         if legends:  # FOR OLD MATPLOTLIB COMPATIBILITY
             plt.savefig(dest_file, bbox_extra_artists=(lgd,), bbox_inches="tight")
         else:
@@ -571,7 +454,6 @@ def plot_acq_func(
         plt.ylim(min(af) - 0.1 * yd, max(af) + 0.1 * yd)
         plt.xlabel(axis_labels[0], size=24)
         plt.ylabel("$acqfn(x)$", size=24)
-        top = 0.99
         if legends:
             lgd = plt.legend(
                 bbox_to_anchor=(0.0, 1.02, 1.0, 0.102),
@@ -581,11 +463,9 @@ def plot_acq_func(
                 borderaxespad=0.0,
                 prop={"size": 20},
             )
-            top = 0.9
         plt.gcf().set_size_inches(10, 8)
         plt.gca().tick_params(labelsize=18)
         plt.gca().ticklabel_format(useOffset=False)
-        # plt.tight_layout()
         if legends:  # FOR OLD MATPLOTLIB COMPATIBILITY
             plt.savefig(dest_file, bbox_extra_artists=(lgd,), bbox_inches="tight")
         else:
@@ -593,77 +473,15 @@ def plot_acq_func(
         plt.close()
 
     elif slice_dim == 2:
-        npts = settings["pp_model_slice"][2]
-        x1 = coords[:, settings["pp_model_slice"][0]]
-        x2 = coords[:, settings["pp_model_slice"][1]]
-
-        plt.contour(x1[:npts], x2[::npts], af.reshape(npts, npts), 25, colors="k")
-        plt.contourf(x1[:npts], x2[::npts], af.reshape(npts, npts), 150, cmap="viridis")
-        cbar = plt.colorbar()  # , orientation='horizontal')
-        cbar.set_label(label="$af(x)$", size=24)
-        cbar.ax.tick_params(labelsize=18)
+        x1, x2, af = _get_2d_arrays(coords, settings, af)
+        _plot_2d_contours(x1, x2, af, label='$af(x)$')
 
         lo = False
-        if acqs is not None:
-            x1 = acqs[:, settings["pp_model_slice"][0]]
-            x2 = acqs[:, settings["pp_model_slice"][1]]
-            sz = np.linspace(200, 500, len(x1))
-            lw = np.linspace(3, 8, len(x1))
-            plt.scatter(
-                x1[0],
-                x2[0],
-                s=sz[int(len(x1) / 2)],
-                linewidth=lw[int(len(x1) / 2)],
-                zorder=10,
-                facecolors="none",
-                edgecolors="magenta",
-                label="acqs",
-            )
-            for i in range(1, len(x1)):
-                plt.scatter(
-                    x1[i],
-                    x2[i],
-                    s=sz[i],
-                    linewidth=lw[i],
-                    zorder=10,
-                    facecolors="none",
-                    edgecolors="magenta",
-                )
-            lo = True
+        lo |= _plot_2d_acqs(acqs, settings)
+        lo |= _plot_2d_xnext(xnext, settings)
+        lo |= _plot_2d_xhat(xhat, settings)
 
-        if xhat is not None:
-            plt.plot(
-                xhat[settings["pp_model_slice"][0]],
-                xhat[settings["pp_model_slice"][1]],
-                "r*",
-                zorder=19,
-                markersize=26,
-                label="$\hat{x}$",
-            )
-            lo = True
-
-        if xnext is not None:
-            plt.plot(
-                xnext[settings["pp_model_slice"][0]],
-                xnext[settings["pp_model_slice"][1]],
-                "b^",
-                zorder=20,
-                markersize=26,
-                label="$x_{next}$",
-            )
-            lo = True
-
-        plt.xlim(
-            min(coords[:, settings["pp_model_slice"][0]]),
-            max(coords[:, settings["pp_model_slice"][0]]),
-        )
-        plt.ylim(
-            min(coords[:, settings["pp_model_slice"][1]]),
-            max(coords[:, settings["pp_model_slice"][1]]),
-        )
-        plt.xlabel(axis_labels[0], size=24)
-        plt.ylabel(axis_labels[1], size=24)
-        top = 0.99
+        _plot_2d_labels_settings(axis_labels)
         if legends and lo:
             lgd = plt.legend(
                 bbox_to_anchor=(0.0, 1.02, 1.0, 0.102),
@@ -673,11 +491,6 @@ def plot_acq_func(
                 borderaxespad=0.0,
                 prop={"size": 20},
             )
-            top = 0.9
-        plt.gcf().set_size_inches(10, 8)
-        plt.gca().tick_params(labelsize=18)
-        plt.gca().ticklabel_format(useOffset=False)
-        # plt.tight_layout()
         if legends and lo:  # FOR OLD MATPLOTLIB COMPATIBILITY
             plt.savefig(dest_file, bbox_extra_artists=(lgd,), bbox_inches="tight")
         else:
@@ -714,31 +527,16 @@ def plot_truef(settings, dest_file, truef_data, axis_labels=None):
         plt.gcf().set_size_inches(10, 8)
         plt.gca().tick_params(labelsize=18)
         plt.gca().ticklabel_format(useOffset=False)
-        # plt.tight_layout()
         plt.tight_layout()
         plt.savefig(dest_file)
         plt.close()
 
     elif slice_dim == 2:
-        npts = settings["pp_truef_npts"]
-        x1 = coords[:, settings["pp_model_slice"][0]]
-        x2 = coords[:, settings["pp_model_slice"][1]]
-
-        plt.contour(x1[:npts], x2[::npts], tf.reshape(npts, npts), 25, colors="k")
-        plt.contourf(x1[:npts], x2[::npts], tf.reshape(npts, npts), 150, cmap="viridis")
-        cbar = plt.colorbar()  # , orientation='horizontal')
-        cbar.set_label(label="$f(x)$", size=24)
-        cbar.ax.tick_params(labelsize=18)
-
-        plt.xlim(min(x1), max(x1))
-        plt.ylim(min(x2), max(x2))
-        plt.xlabel(axis_labels[0], size=24)
-        plt.ylabel(axis_labels[1], size=24)
-        plt.gcf().set_size_inches(10, 8)
-        plt.gca().tick_params(labelsize=18)
-        plt.gca().ticklabel_format(useOffset=False)
+        x1, x2, tf = _get_2d_arrays(coords, settings, tf,
+                                    npts=settings["pp_truef_npts"])
+        _plot_2d_contours(x1, x2, tf, label='$f(x)$')
+        _plot_2d_labels_settings(axis_labels)
         plt.title("True function", size=20)
-        # plt.tight_layout()
         plt.tight_layout()
         plt.savefig(dest_file)
         plt.close()
@@ -746,3 +544,134 @@ def plot_truef(settings, dest_file, truef_data, axis_labels=None):
         raise TypeError(
             "ERROR: True function plot only" + " applicable up to 2D (slices)"
         )
+
+
+def _get_2d_arrays(coords, settings, data, *, npts=None):
+    if npts is None:
+        npts = settings["pp_model_slice"][2]
+    # x1 and x2 here are like from meshgrid()
+    x1 = coords[:, settings["pp_model_slice"][0]].reshape(npts, npts)
+    x2 = coords[:, settings["pp_model_slice"][1]].reshape(npts, npts)
+    reshaped_data = data.reshape(npts, npts)
+    return x1, x2, reshaped_data
+
+
+def _plot_2d_contours(x1, x2, data, *, label='data'):
+    plt.contour(x1, x2, data, 25, colors='k')
+    plt.pcolormesh(x1, x2, data, cmap='viridis', shading='gouraud',
+                   rasterized=True)
+    cbar = plt.colorbar()
+    cbar.set_label(label=label, size=24)
+    cbar.ax.tick_params(labelsize=18)
+    plt.xlim(x1.min(), x1.max())
+    plt.ylim(x2.min(), x2.max())
+
+
+def _plot_2d_xhat(xhat, settings):
+    if xhat is None:
+        return False
+    plt.plot(
+        xhat[settings["pp_model_slice"][0]],
+        xhat[settings["pp_model_slice"][1]],
+        "r*",
+        markersize=26,
+        label="$\hat{x}$",
+        zorder=10,
+    )
+    return True
+
+
+def _plot_2d_acqs(acqs, settings):
+    if acqs is None:
+        return False
+    x1 = acqs[:, settings["pp_model_slice"][0]]
+    x2 = acqs[:, settings["pp_model_slice"][1]]
+    sz = np.linspace(200, 500, len(x1))
+    lw = np.linspace(3, 8, len(x1))
+    plt.scatter(
+        x1[0],
+        x2[0],
+        s=sz[int(len(x1) / 2.0)],
+        linewidth=lw[int(len(x1) / 2.0)],
+        facecolors="none",
+        edgecolors="magenta",
+        label="acqs",
+        zorder=10,
+    )
+    plt.scatter(
+        x1,
+        x2,
+        s=sz,
+        linewidths=lw,
+        facecolors="none",
+        edgecolors="magenta",
+        zorder=10,
+    )
+    return True
+
+
+def _plot_2d_xnext(xnext, settings):
+    if xnext is None:
+        return False
+    plt.plot(
+        xnext[settings["pp_model_slice"][0]],
+        xnext[settings["pp_model_slice"][1]],
+        "b^",
+        markersize=26,
+        label="$x_{next}$",
+        zorder=10,
+    )
+    return True
+
+
+def _plot_2d_minima(minima, settings):
+    if minima is None:
+        return False
+    x1 = minima[:, settings["pp_model_slice"][0]]
+    x2 = minima[:, settings["pp_model_slice"][1]]
+    plt.scatter(
+        x1,
+        x2,
+        s=350,
+        linewidth=6,
+        facecolors="none",
+        edgecolors="navajowhite",
+        label="minima",
+        zorder=10,
+    )
+    return True
+
+
+def _plot_2d_paths(paths, settings):
+    if paths is None:
+        return
+    threshold = min(settings["bounds"][:, 1] - settings["bounds"][:, 0]) / 2
+    for path in paths:
+        start = 0
+        stop = 1
+        while True:
+            if (
+                stop == path.crds.shape[0] - 1
+                or np.linalg.norm(path.crds[stop, :] - path.crds[stop - 1, :])
+                > threshold
+            ):
+                plt.plot(
+                    path.crds[range(start, stop), 0],
+                    path.crds[range(start, stop), 1],
+                    linewidth=3.0,
+                    color="red",
+                )
+                if stop == path.crds.shape[0] - 1:
+                    break
+                else:
+                    start = stop
+            stop += 1
+
+
+def _plot_2d_labels_settings(axis_labels):
+    # XXX one could use mpl.rcParams or styles to set these
+    plt.xlabel(axis_labels[0], size=24)
+    plt.ylabel(axis_labels[1], size=24)
+    plt.gcf().set_size_inches(10, 8)
+    plt.gca().tick_params(labelsize=18)
+    plt.gca().ticklabel_format(useOffset=False)
