@@ -216,7 +216,8 @@ class PPMain:
                     [
                         min_preds[i, 0],
                         euclidean(
-                            min_preds[i - 1, 2 : dim + 2], min_preds[i, 2 : dim + 2],
+                            min_preds[i - 1, 2 : dim + 2],
+                            min_preds[i, 2 : dim + 2],
                         ),
                         abs(min_preds[i - 1, -2] - min_preds[i, -2])
                         / est_yranges[
@@ -272,7 +273,7 @@ class PPMain:
         ioutils.overwrite(
             self.files["hypers"],
             "# Model hyperparameter values by iteration"
-            + " (iter npts variance lengthscales)\n",
+            + " (iter npts parameters)\n",
         )
         # 5. True function values at xhats
         if self.settings["pp_truef_at_xhats"] and min_preds is not None:
@@ -321,7 +322,12 @@ class PPMain:
                     ioutils.append_write(
                         self.files["conv_meas"],
                         ioutils.data_line(
-                            [it, npts, conv_meas[ind_cm, -2], conv_meas[ind_cm, -1],],
+                            [
+                                it,
+                                npts,
+                                conv_meas[ind_cm, -2],
+                                conv_meas[ind_cm, -1],
+                            ],
                             fstr="%18.10E",
                         ),
                     )
@@ -450,7 +456,7 @@ class PPMain:
             if sts["pp_local_minima"] is not None:
                 self.main_output.progress_msg("Finding model local minima", 2)
                 mins = Minimization.minimize(
-                    bo.model.mu_with_grad,
+                    bo.model.predict_mean_grad,
                     sts["bounds"],
                     sts["kernel"],
                     np.hstack([bo.get_x(), bo.get_y()]),
@@ -499,11 +505,13 @@ class PPMain:
                 macqs = acqs[: ind_acqs + 1, 1:]
                 if slc_dim < sts.dim and slc_dim == 1:
                     macqs = None
+
                 ind_xnext = _find_index(xnexts, npts)
-                if ind_xnext is not None and slc_dim == sts.dim:
+                if ind_xnext is not None:
                     xnext = xnexts[ind_xnext, 1:]
                 else:
                     xnext = None
+
                 if sts["pp_local_minima"] is not None and slc_dim == sts.dim:
                     minima = np.atleast_2d(np.array(minima_data)).T
                 else:
